@@ -1,19 +1,24 @@
-'''
-  Copyright European Organization for Nuclear Research (CERN)
-
-  Licensed under the Apache License, Version 2.0 (the "License");
-  You may not use this file except in compliance with the License.
-  You may obtain a copy of the License at
-               http://www.apache.org/licenses/LICENSE-2.0
-
-  Authors:
-  - Angelos Molfetas, <angelos.molfetas@cern.ch>, 2012
-  - Vincent Garonne, <vincent.garonne@cern.ch>, 2012-2017
-  - Mario Lassnig, <mario.lassnig@cern.ch>, 2012
-  - Andrew Lister, <andrew.lister@stfc.ac.uk>, 2019
-
-  PY3K COMPATIBLE
-'''
+# -*- coding: utf-8 -*-
+# Copyright 2012-2022 CERN
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#    http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+#
+# Authors:
+# - Mario Lassnig <mario.lassnig@cern.ch>, 2012-2013
+# - Vincent Garonne <vincent.garonne@cern.ch>, 2012-2017
+# - Hannes Hansen <hannes.jakob.hansen@cern.ch>, 2018
+# - Andrew Lister <andrew.lister@stfc.ac.uk>, 2019
+# - Radu Carpa <radu.carpa@cern.ch>, 2022
 
 from copy import deepcopy
 from rucio.common.types import InternalAccount, InternalScope
@@ -22,22 +27,23 @@ from rucio.core.rse import get_rse_id
 from rucio.common.exception import RSENotFound
 
 
-def has_permission(issuer, action, kwargs, vo='def'):
+def has_permission(issuer, action, kwargs, vo='def', session=None):
     """
     Checks if an account has the specified permission to
     execute an action with parameters.
 
-    :param issuer: The Account issuer.
-    :param vo:     The VO to check against.
-    :param action: The action (API call) called by the account.
-    :param kwargs: List of arguments for the action.
+    :param issuer:  The Account issuer.
+    :param vo:      The VO to check against.
+    :param action:  The action (API call) called by the account.
+    :param session: The db session to use
+    :param kwargs:  List of arguments for the action.
     :returns: True if account is allowed to call the API call, otherwise False
     """
 
     kwargs = deepcopy(kwargs)
     if 'rse' in kwargs and 'rse_id' not in kwargs:
         try:
-            rse_id = get_rse_id(rse=kwargs.get('rse'), vo=vo)
+            rse_id = get_rse_id(rse=kwargs.get('rse'), vo=vo, session=session)
         except RSENotFound:
             rse_id = None
         kwargs.update({'rse_id': rse_id})
@@ -63,4 +69,4 @@ def has_permission(issuer, action, kwargs, vo='def'):
 
     issuer = InternalAccount(issuer, vo=vo)
 
-    return permission.has_permission(issuer=issuer, action=action, kwargs=kwargs)
+    return permission.has_permission(issuer=issuer, action=action, kwargs=kwargs, session=session)
