@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2012-2021 CERN
+# Copyright 2012-2022 CERN
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -21,7 +21,7 @@
 # - Cedric Serfon <cedric.serfon@cern.ch>, 2014-2020
 # - Ralph Vigne <ralph.vigne@cern.ch>, 2015
 # - Joaquín Bogado <jbogado@linti.unlp.edu.ar>, 2015-2018
-# - Martin Barisits <martin.barisits@cern.ch>, 2016-2020
+# - Martin Barisits <martin.barisits@cern.ch>, 2016-2021
 # - Tobias Wegner <twegner@cern.ch>, 2017
 # - Brian Bockelman <bbockelm@cse.unl.edu>, 2017-2018
 # - Robert Illingworth <illingwo@fnal.gov>, 2018
@@ -37,6 +37,10 @@
 # - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020-2021
 # - Radu Carpa <radu.carpa@cern.ch>, 2021
 # - David Población Criado <david.poblacion.criado@cern.ch>, 2021
+# - Paul Millar <paul.millar@desy.de>, 2021
+# - martynia <martynia@users.noreply.github.com>, 2021
+# - Igor Mandrichenko <ivm@fnal.gov>, 2021
+# - Joel Dierkes <joel.dierkes@cern.ch>, 2022
 
 '''
  Client class for callers of the Rucio system
@@ -86,17 +90,6 @@ STATUS_CODES_TO_RETRY = [502, 503, 504]
 MAX_RETRY_BACK_OFF_SECONDS = 10
 
 
-def back_off(retry_number, reason):
-    """
-    Sleep a certain amount of time which increases with the retry count
-    :param retry_number: the retry iteration
-    :param reason: the reason to backoff which will be shown to the user
-    """
-    sleep_time = min(MAX_RETRY_BACK_OFF_SECONDS, 0.25 * 2 ** retry_number)
-    LOG.warning("Waiting {}s due to reason: {} ".format(sleep_time, reason))
-    time.sleep(sleep_time)
-
-
 @REGION.cache_on_arguments(namespace='host_to_choose')
 def choice(hosts):
     """
@@ -117,7 +110,7 @@ class BaseClient(object):
     TOKEN_PREFIX = 'auth_token_'
     TOKEN_EXP_PREFIX = 'auth_token_exp_'
 
-    def __init__(self, rucio_host=None, auth_host=None, account=None, ca_cert=None, auth_type=None, creds=None, timeout=600, user_agent='rucio-clients', vo=None):
+    def __init__(self, rucio_host=None, auth_host=None, account=None, ca_cert=None, auth_type=None, creds=None, timeout=600, user_agent='rucio-clients', vo=None, logger=None):
         """
         Constructor of the BaseClient.
         :param rucio_host: The address of the rucio server, if None it is read from the config file.
@@ -131,11 +124,13 @@ class BaseClient(object):
         :param creds: Dictionary with credentials needed for authentication.
         :param user_agent: Indicates the client.
         :param vo: The VO to authenticate into.
+        :param logger: Logger object to use. If None, use the default LOG created by the module
         """
 
         self.host = rucio_host
         self.list_hosts = []
         self.auth_host = auth_host
+        self.logger = logger or LOG
         self.session = Session()
         self.user_agent = "%s/%s" % (user_agent, version.version_string())  # e.g. "rucio-clients/0.2.13"
         sys.argv[0] = sys.argv[0].split('/')[-1]
@@ -154,7 +149,7 @@ class BaseClient(object):
             self.trace_host = config_get('trace', 'trace_host')
         except (NoOptionError, NoSectionError):
             self.trace_host = self.host
-            LOG.debug('No trace_host passed. Using rucio_host instead')
+            self.logger.debug('No trace_host passed. Using rucio_host instead')
 
         self.account = account
         self.vo = vo
@@ -173,7 +168,7 @@ class BaseClient(object):
         self.auth_oidc_refresh_before_exp = config_get_int('client', 'auth_oidc_refresh_before_exp', False, 20)
 
         if auth_type is None:
-            LOG.debug('No auth_type passed. Trying to get it from the environment variable RUCIO_AUTH_TYPE and config file.')
+            self.logger.debug('No auth_type passed. Trying to get it from the environment variable RUCIO_AUTH_TYPE and config file.')
             if 'RUCIO_AUTH_TYPE' in environ:
                 if environ['RUCIO_AUTH_TYPE'] not in ['userpass', 'x509', 'x509_proxy', 'gss', 'ssh', 'saml', 'oidc']:
                     raise MissingClientParameter('Possible RUCIO_AUTH_TYPE values: userpass, x509, x509_proxy, gss, ssh, saml, oidc, vs. ' + environ['RUCIO_AUTH_TYPE'])
@@ -207,23 +202,33 @@ class BaseClient(object):
                 self.creds['oidc_polling'] = config_get_bool('client', 'oidc_polling', False, False)
 
         if not self.creds:
-            LOG.debug('No creds passed. Trying to get it from the config file.')
+            self.logger.debug('No creds passed. Trying to get it from the config file.')
             self.creds = {}
             try:
                 if self.auth_type in ['userpass', 'saml']:
                     self.creds['username'] = config_get('client', 'username')
                     self.creds['password'] = config_get('client', 'password')
                 elif self.auth_type == 'x509':
-                    self.creds['client_cert'] = path.abspath(path.expanduser(path.expandvars(config_get('client', 'client_cert'))))
+                    if "RUCIO_CLIENT_CERT" in environ:
+                        client_cert = environ["RUCIO_CLIENT_CERT"]
+                    else:
+                        client_cert = config_get('client', 'client_cert')
+                    self.creds['client_cert'] = path.abspath(path.expanduser(path.expandvars(client_cert)))
                     if not path.exists(self.creds['client_cert']):
                         raise MissingClientParameter('X.509 client certificate not found: %s' % self.creds['client_cert'])
-                    self.creds['client_key'] = path.abspath(path.expanduser(path.expandvars(config_get('client', 'client_key'))))
+
+                    if "RUCIO_CLIENT_KEY" in environ:
+                        client_key = environ["RUCIO_CLIENT_KEY"]
+                    else:
+                        client_key = config_get('client', 'client_key')
+                    self.creds['client_key'] = path.abspath(path.expanduser(path.expandvars(client_key)))
                     if not path.exists(self.creds['client_key']):
                         raise MissingClientParameter('X.509 client key not found: %s' % self.creds['client_key'])
                     else:
                         perms = oct(os.stat(self.creds['client_key']).st_mode)[-3:]
                         if perms not in ['400', '600']:
                             raise CannotAuthenticate('X.509 authentication selected, but private key (%s) permissions are liberal (required: 400 or 600, found: %s)' % (self.creds['client_key'], perms))
+
                 elif self.auth_type == 'x509_proxy':
                     try:
                         self.creds['client_proxy'] = path.abspath(path.expanduser(path.expandvars(config_get('client', 'client_x509_proxy'))))
@@ -257,20 +262,20 @@ class BaseClient(object):
             raise ClientProtocolNotSupported('\'%s\' not supported' % auth_scheme)
 
         if (rucio_scheme == 'https' or auth_scheme == 'https') and ca_cert is None:
-            LOG.debug('HTTPS is required, but no ca_cert was passed. Trying to get it from X509_CERT_DIR.')
+            self.logger.debug('HTTPS is required, but no ca_cert was passed. Trying to get it from X509_CERT_DIR.')
             self.ca_cert = os.environ.get('X509_CERT_DIR', None)
             if self.ca_cert is None:
-                LOG.debug('HTTPS is required, but no ca_cert was passed and X509_CERT_DIR is not defined. Trying to get it from the config file.')
+                self.logger.debug('HTTPS is required, but no ca_cert was passed and X509_CERT_DIR is not defined. Trying to get it from the config file.')
                 try:
                     self.ca_cert = path.expandvars(config_get('client', 'ca_cert'))
                 except (NoOptionError, NoSectionError):
-                    LOG.debug('No ca_cert found in configuration. Falling back to Mozilla default CA bundle (certifi).')
+                    self.logger.debug('No ca_cert found in configuration. Falling back to Mozilla default CA bundle (certifi).')
                     self.ca_cert = True
 
         self.list_hosts = [self.host]
 
         if account is None:
-            LOG.debug('No account passed. Trying to get it from the RUCIO_ACCOUNT environment variable or the config file.')
+            self.logger.debug('No account passed. Trying to get it from the RUCIO_ACCOUNT environment variable or the config file.')
             try:
                 self.account = environ['RUCIO_ACCOUNT']
             except KeyError:
@@ -280,15 +285,15 @@ class BaseClient(object):
                     pass
 
         if vo is None:
-            LOG.debug('No VO passed. Trying to get it from environment variable RUCIO_VO.')
+            self.logger.debug('No VO passed. Trying to get it from environment variable RUCIO_VO.')
             try:
                 self.vo = environ['RUCIO_VO']
             except KeyError:
-                LOG.debug('No VO found. Trying to get it from the config file.')
+                self.logger.debug('No VO found. Trying to get it from the config file.')
                 try:
                     self.vo = config_get('client', 'vo')
                 except (NoOptionError, NoSectionError):
-                    LOG.debug('No VO found. Using default VO.')
+                    self.logger.debug('No VO found. Using default VO.')
                     self.vo = 'def'
 
         token_filename_suffix = "for_default_account" if self.account is None else "for_account_" + self.account
@@ -309,10 +314,10 @@ class BaseClient(object):
 
         try:
             self.request_retries = int(config_get('client', 'request_retries'))
-        except NoOptionError:
+        except (NoOptionError, RuntimeError):
             LOG.debug('request_retries not specified in config file. Taking default.')
         except ValueError:
-            LOG.debug('request_retries must be an integer. Taking default.')
+            self.logger.debug('request_retries must be an integer. Taking default.')
 
     def _get_exception(self, headers, status_code=None, data=None):
         """
@@ -361,14 +366,30 @@ class BaseClient(object):
             if response.text:
                 yield response.text
 
-    def _send_request(self, url, headers=None, type='GET', data=None, params=None, stream=False, get_token=False,
+    def _reduce_data(self, data, maxlen=132):
+        text = data if isinstance(data, str) else data.decode("utf-8")
+        if len(text) > maxlen:
+            text = "%s ... %s" % (text[:maxlen - 15], text[-10:])
+        return text
+
+    def _back_off(self, retry_number, reason):
+        """
+        Sleep a certain amount of time which increases with the retry count
+        :param retry_number: the retry iteration
+        :param reason: the reason to backoff which will be shown to the user
+        """
+        sleep_time = min(MAX_RETRY_BACK_OFF_SECONDS, 0.25 * 2 ** retry_number)
+        self.logger.warning("Waiting {}s due to reason: {} ".format(sleep_time, reason))
+        time.sleep(sleep_time)
+
+    def _send_request(self, url, headers=None, type_='GET', data=None, params=None, stream=False, get_token=False,
                       cert=None, auth=None, verify=None):
         """
         Helper method to send requests to the rucio server. Gets a new token and retries if an unauthorized error is returned.
 
         :param url: the http url to use.
         :param headers: additional http headers to send.
-        :param type: the http request type to use.
+        :param type_: the http request type to use.
         :param data: post data.
         :param params: (optional) Dictionary or bytes to be sent in the url query string.
         :param get_token: (optional) if it is called from a _get_token function.
@@ -391,24 +412,38 @@ class BaseClient(object):
         if verify is None:
             verify = self.ca_cert
 
+        self.logger.debug("HTTP request: %s %s" % (type_, url))
+        for h, v in hds.items():
+            if h == 'X-Rucio-Auth-Token':
+                v = "[hidden]"
+            self.logger.debug("HTTP header:  %s: %s" % (h, v))
+        if type_ != "GET" and data:
+            text = self._reduce_data(data)
+            self.logger.debug("Request data (length=%d): [%s]" % (len(data), text))
+
         result = None
         for retry in range(self.AUTH_RETRIES + 1):
             try:
-                if type == 'GET':
+                if type_ == 'GET':
                     result = self.session.get(url, headers=hds, verify=verify, timeout=self.timeout, params=params, stream=True, cert=cert, auth=auth)
-                elif type == 'PUT':
+                elif type_ == 'PUT':
                     result = self.session.put(url, headers=hds, data=data, verify=verify, timeout=self.timeout)
-                elif type == 'POST':
+                elif type_ == 'POST':
                     result = self.session.post(url, headers=hds, data=data, verify=verify, timeout=self.timeout, stream=stream)
-                elif type == 'DEL':
+                elif type_ == 'DEL':
                     result = self.session.delete(url, headers=hds, data=data, verify=verify, timeout=self.timeout)
                 else:
-                    return
+                    self.logger.debug("Unknown request type %s. Request was not sent" % (type_,))
+                    return None
+                self.logger.debug("HTTP Response: %s %s" % (result.status_code, result.reason))
                 if result.status_code in STATUS_CODES_TO_RETRY:
-                    back_off(retry, reason='server returned {}'.format(result.status_code))
+                    self._back_off(retry, 'server returned {}'.format(result.status_code))
                     continue
+                if result.status_code // 100 != 2 and result.text:
+                    # do not do this for successful requests because the caller may be expecting streamed response
+                    self.logger.debug("Response text (length=%d): [%s]" % (len(result.text), result.text))
             except ConnectionError as error:
-                LOG.error('ConnectionError: ' + str(error))
+                self.logger.error('ConnectionError: ' + str(error))
                 if retry > self.request_retries:
                     raise
                 continue
@@ -417,7 +452,7 @@ class BaseClient(object):
                 # While in python3 we can directly catch 'BrokenPipeError', in python2 it doesn't exist.
                 if getattr(error, 'errno') != errno.EPIPE:
                     raise
-                LOG.error('BrokenPipe: ' + str(error))
+                self.logger.error('BrokenPipe: ' + str(error))
                 if retry > self.request_retries:
                     raise
                 continue
@@ -457,7 +492,7 @@ class BaseClient(object):
                         raise CannotAuthenticate(result.headers["ExceptionClass"])
                 elif result.text:
                     raise CannotAuthenticate(result.text)
-            LOG.error('Cannot retrieve authentication token!')
+            self.logger.error('Cannot retrieve authentication token!')
             return False
 
         if result.status_code != codes.ok:  # pylint: disable-msg=E1101
@@ -508,14 +543,14 @@ class BaseClient(object):
                 new_token = refresh_result.headers['X-Rucio-Auth-Token']
                 new_exp_epoch = refresh_result.headers['X-Rucio-Auth-Token-Expires']
                 if new_token and new_exp_epoch:
-                    LOG.debug("Saving token %s and expiration epoch %s to files" % (str(new_token), str(new_exp_epoch)))
+                    self.logger.debug("Saving token %s and expiration epoch %s to files" % (str(new_token), str(new_exp_epoch)))
                     # save to the file
                     self.auth_token = new_token
                     self.token_exp_epoch = new_exp_epoch
                     self.__write_token()
                     self.headers['X-Rucio-Auth-Token'] = self.auth_token
                     return True
-                LOG.debug("No new token was received, possibly invalid/expired \
+                self.logger.debug("No new token was received, possibly invalid/expired \
                            \ntoken or a token with no refresh token in Rucio DB")
                 return False
         else:
@@ -551,9 +586,9 @@ class BaseClient(object):
         result = None
         request_auth_url = build_url(self.auth_host, path='auth/oidc')
         # requesting authorization URL specific to the user & Rucio OIDC Client
-        LOG.debug("Initial auth URL request headers %s to files" % str(headers))
+        self.logger.debug("Initial auth URL request headers %s to files" % str(headers))
         OIDC_auth_res = self._send_request(request_auth_url, headers=headers, get_token=True)
-        LOG.debug("Response headers %s and text %s" % (str(OIDC_auth_res.headers), str(OIDC_auth_res.text)))
+        self.logger.debug("Response headers %s and text %s" % (str(OIDC_auth_res.headers), str(OIDC_auth_res.text)))
         # with the obtained authorization URL we will contact the Identity Provider to get to the login page
         if 'X-Rucio-OIDC-Auth-URL' not in OIDC_auth_res.headers:
             print("Rucio Client did not succeed to get AuthN/Z URL from the Rucio Auth Server. \
@@ -605,12 +640,12 @@ class BaseClient(object):
             # getting the login URL and logging in the user
             login_url = auth_res.url
             start = time.time()
-            result = self._send_request(login_url, type='POST', data=userpass)
+            result = self._send_request(login_url, type_='POST', data=userpass)
 
             # if the Rucio OIDC Client configuration does not match the one registered at the Identity Provider
             # the user will get an OAuth error
             if 'OAuth Error' in result.text:
-                LOG.error('Identity Provider does not allow to proceed. Could be due \
+                self.logger.error('Identity Provider does not allow to proceed. Could be due \
                            \nto misconfigured redirection server name of the Rucio OIDC Client.')
                 return False
             # In case Rucio Client is not authorized to request information about this user yet,
@@ -624,13 +659,13 @@ class BaseClient(object):
                                 "authorize": "Authorize"}
                 form_data.update(default_data)
                 print('Automatically authorising request of the following info on behalf of user: %s', str(form_data))
-                LOG.warning('Automatically authorising request of the following info on behalf of user: %s',
-                            str(form_data))
+                self.logger.warning('Automatically authorising request of the following info on behalf of user: %s',
+                                    str(form_data))
                 # authorizing info request on behalf of the user until he/she revokes this authorization !
-                result = self._send_request(result.url, type='POST', data=form_data)
+                result = self._send_request(result.url, type_='POST', data=form_data)
 
         if not result or 'result' not in locals():
-            LOG.error('Cannot retrieve authentication token!')
+            self.logger.error('Cannot retrieve authentication token!')
             return False
 
         if result.status_code != codes.ok:  # pylint: disable-msg=E1101
@@ -641,7 +676,7 @@ class BaseClient(object):
 
         self.auth_token = result.headers['x-rucio-auth-token']
         if self.auth_oidc_refresh_active:
-            LOG.debug("Reseting the token expiration epoch file content.")
+            self.logger.debug("Reseting the token expiration epoch file content.")
             # reset the token expiration epoch file content
             # at new CLI OIDC authentication
             self.token_exp_epoch = None
@@ -670,10 +705,10 @@ class BaseClient(object):
             client_cert = self.creds['client_proxy']
 
         if not path.exists(client_cert):
-            LOG.error('given client cert (%s) doesn\'t exist' % client_cert)
+            self.logger.error('given client cert (%s) doesn\'t exist' % client_cert)
             return False
         if client_key is not None and not path.exists(client_key):
-            LOG.error('given client key (%s) doesn\'t exist' % client_key)
+            self.logger.error('given client key (%s) doesn\'t exist' % client_key)
 
         if client_key is None:
             cert = client_cert
@@ -685,7 +720,7 @@ class BaseClient(object):
         # Note a response object for a failed request evaluates to false, so we cannot
         # use "not result" here
         if result is None:
-            LOG.error('Internal error: Request for authentication token returned no result!')
+            self.logger.error('Internal error: Request for authentication token returned no result!')
             return False
 
         if result.status_code != codes.ok:   # pylint: disable-msg=E1101
@@ -707,10 +742,10 @@ class BaseClient(object):
 
         private_key_path = self.creds['ssh_private_key']
         if not path.exists(private_key_path):
-            LOG.error('given private key (%s) doesn\'t exist' % private_key_path)
+            self.logger.error('given private key (%s) doesn\'t exist' % private_key_path)
             return False
         if private_key_path is not None and not path.exists(private_key_path):
-            LOG.error('given private key (%s) doesn\'t exist' % private_key_path)
+            self.logger.error('given private key (%s) doesn\'t exist' % private_key_path)
             return False
 
         url = build_url(self.auth_host, path='auth/ssh_challenge_token')
@@ -718,7 +753,7 @@ class BaseClient(object):
         result = self._send_request(url, get_token=True)
 
         if not result:
-            LOG.error('cannot get ssh_challenge_token')
+            self.logger.error('cannot get ssh_challenge_token')
             return False
 
         if result.status_code != codes.ok:   # pylint: disable-msg=E1101
@@ -728,7 +763,7 @@ class BaseClient(object):
             raise exc_cls(exc_msg)
 
         self.ssh_challenge_token = result.headers['x-rucio-ssh-challenge-token']
-        LOG.debug('got new ssh challenge token \'%s\'' % self.ssh_challenge_token)
+        self.logger.debug('got new ssh challenge token \'%s\'' % self.ssh_challenge_token)
 
         # sign the challenge token with the private key
         with open(private_key_path, 'r') as fd_private_key_path:
@@ -741,7 +776,7 @@ class BaseClient(object):
         result = self._send_request(url, headers=headers, get_token=True)
 
         if not result:
-            LOG.error('Cannot retrieve authentication token!')
+            self.logger.error('Cannot retrieve authentication token!')
             return False
 
         if result.status_code != codes.ok:   # pylint: disable-msg=E1101
@@ -767,7 +802,7 @@ class BaseClient(object):
         result = self._send_request(url, get_token=True, auth=HTTPKerberosAuth())
 
         if not result:
-            LOG.error('Cannot retrieve authentication token!')
+            self.logger.error('Cannot retrieve authentication token!')
             return False
 
         if result.status_code != codes.ok:   # pylint: disable-msg=E1101
@@ -793,11 +828,11 @@ class BaseClient(object):
         if SAML_auth_result.headers['X-Rucio-Auth-Token']:
             return SAML_auth_result.headers['X-Rucio-Auth-Token']
         SAML_auth_url = SAML_auth_result.headers['X-Rucio-SAML-Auth-URL']
-        result = self._send_request(SAML_auth_url, type='POST', data=userpass, verify=False)
+        result = self._send_request(SAML_auth_url, type_='POST', data=userpass, verify=False)
         result = self._send_request(url, get_token=True)
 
         if not result or 'result' not in locals():
-            LOG.error('Cannot retrieve authentication token!')
+            self.logger.error('Cannot retrieve authentication token!')
             return False
 
         if result.status_code != codes.ok:  # pylint: disable-msg=E1101
@@ -814,7 +849,7 @@ class BaseClient(object):
         Calls the corresponding method to receive an auth token depending on the auth type. To be used if a 401 - Unauthorized error is received.
         """
 
-        LOG.debug('get a new token')
+        self.logger.debug('get a new token')
         for retry in range(self.AUTH_RETRIES + 1):
             if self.auth_type == 'userpass':
                 if not self.__get_token_userpass():
@@ -870,7 +905,7 @@ class BaseClient(object):
             raise
         if self.auth_oidc_refresh_active and self.auth_type == 'oidc':
             self.__refresh_token_OIDC()
-        LOG.debug('got token from file')
+        self.logger.debug('got token from file')
         return True
 
     def __write_token(self):
@@ -880,7 +915,7 @@ class BaseClient(object):
         # check if rucio temp directory is there. If not create it with permissions only for the current user
         if not path.isdir(self.token_path):
             try:
-                LOG.debug('rucio token folder \'%s\' not found. Create it.' % self.token_path)
+                self.logger.debug('rucio token folder \'%s\' not found. Create it.' % self.token_path)
                 makedirs(self.token_path, 0o700)
             except Exception:
                 raise
