@@ -40,14 +40,14 @@ from dogpile.cache.api import NO_VALUE
 
 from rucio.common import utils
 from rucio.common.cache import make_region_memcached
-from rucio.common.config import config_get, config_get_bool
+from rucio.common.config import config_get, config_get_bool, config_get_int
 from rucio.common.exception import InvalidRSEExpression
 from rucio.core.rse_expression_parser import parse_expression
 
 if TYPE_CHECKING:
     from typing import Dict, List, Optional
 
-REGION = make_region_memcached(expiration_time=1800, function_key_generator=utils.my_key_generator)
+REGION = make_region_memcached(expiration_time=900, function_key_generator=utils.my_key_generator)
 
 # This product uses GeoLite data created by MaxMind,
 # available from <a href="http://www.maxmind.com">http://www.maxmind.com</a>
@@ -75,7 +75,9 @@ def __download_geoip_db(destination):
     download_url = config_get('core', 'geoip_download_url', raise_exception=False, default=None)
     verify_tls = config_get_bool('core', 'geoip_download_verify_tls', raise_exception=False, default=True)
     if not download_url:
-        licence_key = config_get('core', 'geoip_licence_key', raise_exception=False, default='NOLICENCE')
+        licence_key = config_get('core', 'geoip_licence_key', raise_exception=False, default=None)
+        if not licence_key:
+            raise Exception('Cannot download GeoIP database: licence key not provided')
         download_url = 'https://download.maxmind.com/app/geoip_download?edition_id=%s&license_key=%s&suffix=tar.gz' % (edition_id, licence_key)
 
     result = requests.get(download_url, stream=True, verify=verify_tls)
@@ -94,13 +96,13 @@ def __download_geoip_db(destination):
 
 def __geoip_db():
     db_path = Path(f'/tmp/{GEOIP_DB_EDITION}.mmdb')
-    db_expire_delay = timedelta(days=30)
+    db_expire_delay = timedelta(days=config_get_int('core', 'geoip_expire_delay', raise_exception=False, default=30))
 
     must_download = False
     if not db_path.is_file():
         print('%s does not exist. Downloading it.' % db_path)
         must_download = True
-    elif datetime.fromtimestamp(db_path.stat().st_mtime) < datetime.now() - db_expire_delay:
+    elif db_expire_delay and datetime.fromtimestamp(db_path.stat().st_mtime) < datetime.now() - db_expire_delay:
         print('%s is too old. Re-downloading it.' % db_path)
         must_download = True
 
@@ -136,7 +138,7 @@ def __get_distance(se1, client_location, ignore_error):
     # does not cache ignore_error, str.lower on hostnames/ips is fine
     canonical_parties = list(map(lambda x: str(x).lower(), [se1, client_location['ip'], client_location.get('latitude', ''), client_location.get('longitude', '')]))
     canonical_parties.sort()
-    cache_key = f'replica_sorter:__get_distance|site_distance|{"".join(canonical_parties)}'
+    cache_key = f'replica_sorter:__get_distance|site_distance|{"".join(canonical_parties)}'.replace(' ', '.')
     cache_val = REGION.get(cache_key)
     if cache_val is NO_VALUE:
         try:
