@@ -18,7 +18,7 @@
 # - Ralph Vigne <ralph.vigne@cern.ch>, 2012-2015
 # - Mario Lassnig <mario.lassnig@cern.ch>, 2012-2021
 # - Martin Barisits <martin.barisits@cern.ch>, 2013-2020
-# - Cedric Serfon <cedric.serfon@cern.ch>, 2013-2021
+# - Cedric Serfon <cedric.serfon@cern.ch>, 2013-2022
 # - Thomas Beermann <thomas.beermann@cern.ch>, 2014-2017
 # - Wen Guan <wen.guan@cern.ch>, 2015-2016
 # - Brian Bockelman <bbockelm@cse.unl.edu>, 2018
@@ -39,6 +39,7 @@
 # - Joel Dierkes <joel.dierkes@cern.ch>, 2021
 # - David Población Criado <david.poblacion.criado@cern.ch>, 2021
 
+from datetime import datetime
 import json
 from io import StringIO
 from re import match
@@ -51,7 +52,7 @@ from six import string_types
 from sqlalchemy.exc import DatabaseError, IntegrityError, OperationalError
 from sqlalchemy.orm import aliased
 from sqlalchemy.orm.exc import FlushError
-from sqlalchemy.sql.expression import or_, false
+from sqlalchemy.sql.expression import or_, false, func, case
 
 import rucio.core.account_counter
 from rucio.common import exception, utils
@@ -60,7 +61,7 @@ from rucio.common.config import get_lfn2pfn_algorithm_default
 from rucio.common.utils import CHECKSUM_KEY, is_checksum_valid, GLOBALLY_SUPPORTED_CHECKSUMS
 from rucio.core.rse_counter import add_counter, get_counter
 from rucio.db.sqla import models
-from rucio.db.sqla.constants import RSEType
+from rucio.db.sqla.constants import (RSEType, ReplicaState)
 from rucio.db.sqla.session import read_session, transactional_session, stream_session
 
 if TYPE_CHECKING:
@@ -919,13 +920,15 @@ def add_protocol(rse_id, parameter, session=None):
             for op in parameter['domains'][s]:
                 if op not in utils.rse_supported_protocol_operations():
                     raise exception.RSEOperationNotSupported('Operation \'%s\' not defined in schema.' % (op))
-                op_name = op if op == 'third_party_copy' else ''.join([op, '_', s]).lower()
+                op_name = op if op.startswith('third_party_copy') else ''.join([op, '_', s]).lower()
                 if parameter['domains'][s][op] < 0:
                     raise exception.RSEProtocolPriorityError('The provided priority (%s)for operation \'%s\' in domain \'%s\' is not supported.' % (parameter['domains'][s][op], op, s))
                 parameter[op_name] = parameter['domains'][s][op]
         del parameter['domains']
 
-    # This is needed for forward-compatibility with the 1.28 version, which relies on _read/_write columns
+    # If third_party_copy is set, but not the third_party_copy_read/write,
+    # set the _read/_write by copying the value from the third_party_copy
+    # This is done to maintain compatibility with old scripts.
     if parameter.get('third_party_copy') is not None and parameter.get('third_party_copy_read') is None and parameter.get('third_party_copy_write') is None:
         parameter['third_party_copy_read'] = parameter['third_party_copy']
         parameter['third_party_copy_write'] = parameter['third_party_copy']
@@ -1039,6 +1042,8 @@ def get_rse_protocols(rse_id, schemes=None, session=None):
                           models.RSEProtocols.write_wan,
                           models.RSEProtocols.delete_wan,
                           models.RSEProtocols.third_party_copy,
+                          models.RSEProtocols.third_party_copy_read,
+                          models.RSEProtocols.third_party_copy_write,
                           models.RSEProtocols.extended_attributes).filter(*terms)
 
     for row in query:
@@ -1054,7 +1059,9 @@ def get_rse_protocols(rse_id, schemes=None, session=None):
                  'wan': {'read': row.read_wan,
                          'write': row.write_wan,
                          'delete': row.delete_wan,
-                         'third_party_copy': row.third_party_copy}
+                         'third_party_copy': row.third_party_copy,
+                         'third_party_copy_read': row.third_party_copy_read,
+                         'third_party_copy_write': row.third_party_copy_write}
              },
              'extended_attributes': row.extended_attributes}
 
@@ -1102,7 +1109,7 @@ def update_protocols(rse_id, scheme, data, hostname, port, session=None):
                 if op not in utils.rse_supported_protocol_operations():
                     raise exception.RSEOperationNotSupported('Operation \'%s\' not defined in schema.' % (op))
                 op_name = op
-                if op != 'third_party_copy':
+                if not op.startswith('third_party_copy'):
                     op_name = ''.join([op, '_', s])
                 no = session.query(models.RSEProtocols).\
                     filter(sqlalchemy.and_(models.RSEProtocols.rse_id == rse_id,
@@ -1139,7 +1146,7 @@ def update_protocols(rse_id, scheme, data, hostname, port, session=None):
         for domain in utils.rse_supported_protocol_domains():
             for op in utils.rse_supported_protocol_operations():
                 op_name = op
-                if op != 'third_party_copy':
+                if not op.startswith('third_party_copy'):
                     op_name = ''.join([op, '_', domain])
                 if op_name in data:
                     prots = []
@@ -1403,3 +1410,27 @@ def list_qos_policies(rse_id, session=None):
         raise exception.RucioException(error.args)
 
     return qos_policies
+
+
+@transactional_session
+def fill_rse_expired(rse_id, session=None):
+    """
+    Fill the rse_usage for source expired
+
+    :param rse_id: The RSE id.
+
+    :returns: True if successful, except otherwise.
+    """
+    none_value = None  # Hack to get pep8 happy...
+    query = session.query(func.sum(models.RSEFileAssociation.bytes).label("bytes"), func.count().label("length")).\
+        with_hint(models.RSEFileAssociation, "INDEX_RS_ASC(replicas REPLICAS_TOMBSTONE_IDX)  NO_INDEX_FFS(replicas REPLICAS_TOMBSTONE_IDX)", 'oracle').\
+        filter(models.RSEFileAssociation.tombstone < datetime.utcnow()).\
+        filter(models.RSEFileAssociation.lock_cnt == 0).\
+        filter(case([(models.RSEFileAssociation.tombstone != none_value, models.RSEFileAssociation.rse_id), ]) == rse_id).\
+        filter(or_(models.RSEFileAssociation.state.in_((ReplicaState.AVAILABLE, ReplicaState.UNAVAILABLE, ReplicaState.BAD))))
+    result = query.all()
+    sum_bytes, sum_files = result[0]
+    models.RSEUsage(rse_id=rse_id,
+                    used=sum_bytes,
+                    files=sum_files,
+                    source='expired').save(session=session)
