@@ -27,6 +27,7 @@
 import datetime
 import logging
 import re
+from configparser import NoOptionError, NoSectionError
 
 from json import dumps
 
@@ -35,15 +36,15 @@ from sqlalchemy.exc import IntegrityError, StatementError
 from sqlalchemy.orm import aliased
 from sqlalchemy.orm.exc import NoResultFound
 
-from rucio.common.exception import SubscriptionNotFound, SubscriptionDuplicate, RucioException, ConfigNotFound
-from rucio.core.config import get
+from rucio.common.config import config_get
+from rucio.common.exception import SubscriptionNotFound, SubscriptionDuplicate, RucioException
 from rucio.db.sqla import models
 from rucio.db.sqla.constants import SubscriptionState
 from rucio.db.sqla.session import transactional_session, stream_session, read_session
 
 
 @transactional_session
-def add_subscription(name, account, filter, replication_rules, comments, lifetime, retroactive, dry_run, priority=3, session=None):
+def add_subscription(name, account, filter_, replication_rules, comments, lifetime, retroactive, dry_run, priority=3, session=None):
     """
     Adds a new subscription which will be verified against every new added file and dataset
 
@@ -51,9 +52,9 @@ def add_subscription(name, account, filter, replication_rules, comments, lifetim
     :type account:             String
     :param name:               Name of the subscription
     :type name:                String
-    :param filter:             Dictionary of attributes by which the input data should be filtered
+    :param filter_:             Dictionary of attributes by which the input data should be filtered
                                **Example**: ``{'dsn': 'data11_hi*.express_express.*,data11_hi*physics_MinBiasOverlay*', 'account': 'tzero'}``
-    :type filter:              Dict
+    :type filter_:              Dict
     :param replication_rules:  Replication rules to be set : Dictionary with keys copies, rse_expression, weight, rse_expression
     :type replication_rules:   Dict
     :param comments:           Comments for the subscription
@@ -71,8 +72,8 @@ def add_subscription(name, account, filter, replication_rules, comments, lifetim
     :returns:                  The subscriptionid
     """
     try:
-        keep_history = get('subscriptions', 'keep_history')
-    except ConfigNotFound:
+        keep_history = config_get('subscriptions', 'keep_history')
+    except (NoOptionError, NoSectionError, RuntimeError):
         keep_history = False
 
     SubscriptionHistory = models.SubscriptionHistory
@@ -84,7 +85,7 @@ def add_subscription(name, account, filter, replication_rules, comments, lifetim
     if lifetime:
         lifetime = datetime.datetime.utcnow() + datetime.timedelta(days=lifetime)
     new_subscription = models.Subscription(name=name,
-                                           filter=filter,
+                                           filter=filter_,
                                            account=account,
                                            replication_rules=replication_rules,
                                            state=state,
@@ -135,8 +136,8 @@ def update_subscription(name, account, metadata=None, session=None):
     :raises: SubscriptionNotFound if subscription is not found
     """
     try:
-        keep_history = get('subscriptions', 'keep_history')
-    except ConfigNotFound:
+        keep_history = config_get('subscriptions', 'keep_history')
+    except (NoOptionError, NoSectionError, RuntimeError):
         keep_history = False
     values = {'state': SubscriptionState.UPDATED}
     if 'filter' in metadata and metadata['filter']:
