@@ -88,6 +88,7 @@ def run_once(bulk, db_bulk, suspicious_patterns, retry_protocol_mismatches, hear
                                      total_workers=total_workers,
                                      worker_number=worker_number,
                                      mode_all=True,
+                                     include_dependent=False,
                                      hash_variable='rule_id')
         record_timer('daemons.conveyor.finisher.get_next', (time.time() - time1) * 1000)
         time2 = time.time()
@@ -408,7 +409,7 @@ def __update_bulk_replicas(replicas, session=None, logger=logging.log):
 
     for replica in replicas:
         if not replica['archived']:
-            request_core.archive_request(replica['request_id'], session=session, logger=logger)
+            request_core.archive_request(replica['request_id'], session=session)
         logger(logging.INFO, "HANDLED REQUEST %s DID %s:%s AT RSE %s STATE %s", replica['request_id'], replica['scope'], replica['name'], replica['rse_id'], str(replica['state']))
     return True
 
@@ -427,7 +428,7 @@ def __update_replica(replica, session=None, logger=logging.log):
     try:
         replica_core.update_replicas_states([replica], nowait=True, session=session)
         if not replica['archived']:
-            request_core.archive_request(replica['request_id'], session=session, logger=logger)
+            request_core.archive_request(replica['request_id'], session=session)
         logger(logging.INFO, "HANDLED REQUEST %s DID %s:%s AT RSE %s STATE %s", replica['request_id'], replica['scope'], replica['name'], replica['rse_id'], str(replica['state']))
     except (UnsupportedOperation, ReplicaNotFound) as error:
         logger(logging.WARNING, "ERROR WHEN HANDLING REQUEST %s DID %s:%s AT RSE %s STATE %s: %s", replica['request_id'], replica['scope'], replica['name'], replica['rse_id'], str(replica['state']), str(error))
@@ -445,7 +446,7 @@ def __update_replica(replica, session=None, logger=logging.log):
                                          tombstone=datetime.datetime.utcnow(),
                                          session=session)
             if not replica['archived']:
-                request_core.archive_request(replica['request_id'], session=session, logger=logger)
+                request_core.archive_request(replica['request_id'], session=session)
             logger(logging.INFO, "HANDLED REQUEST %s DID %s:%s AT RSE %s STATE %s", replica['request_id'], replica['scope'], replica['name'], replica['rse_id'], str(replica['state']))
         except Exception as error:
             logger(logging.ERROR, 'Cannot register replica for DID %s:%s at RSE %s - potential dark data - %s', replica['scope'], replica['name'], replica['rse_id'], str(error))
