@@ -51,9 +51,8 @@ from rucio.common import exception
 from rucio.common.config import config_get, config_get_bool, config_get_int
 from rucio.common.logging import setup_logging
 from rucio.common.schema import get_schema_value
-from rucio.core import transfer as transfer_core
 from rucio.core.monitor import MultiCounter, record_timer
-from rucio.daemons.conveyor.common import submit_transfer, get_conveyor_rses, run_conveyor_daemon
+from rucio.daemons.conveyor.common import submit_transfer, get_conveyor_rses, run_conveyor_daemon, next_transfers_to_submit
 from rucio.db.sqla.constants import RequestType
 from rucio.transfertool.fts3 import FTS3Transfertool
 from rucio.transfertool.globus import GlobusTransferTool
@@ -81,7 +80,7 @@ def run_once(bulk, group_bulk, filter_transfertool, transfertool, ignore_availab
     worker_number, total_workers, logger = heartbeat_handler.live()
 
     start_time = time.time()
-    transfers = transfer_core.next_transfers_to_submit(
+    transfers = next_transfers_to_submit(
         total_workers=total_workers,
         worker_number=worker_number,
         partition_hash_var=partition_hash_var,
