@@ -25,6 +25,7 @@
 # - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020-2021
 # - David Población Criado <david.poblacion.criado@cern.ch>, 2021
 # - Radu Carpa <radu.carpa@cern.ch>, 2021
+# - Joel Dierkes <joel.dierkes@cern.ch>, 2021
 
 '''
    Hermes is a daemon to deliver messages: to a messagebroker via STOMP, or emails via SMTP.
@@ -48,7 +49,7 @@ from sqlalchemy.orm.exc import NoResultFound
 import rucio.db.sqla.util
 from rucio.common import exception
 from rucio.common.config import config_get, config_get_int, config_get_bool
-from rucio.common.logging import setup_logging
+from rucio.common.logging import setup_logging, formatted_logger
 from rucio.common.utils import daemon_sleep
 from rucio.core.heartbeat import live, die, sanity_check
 from rucio.core.message import retrieve_messages, delete_messages
@@ -80,16 +81,19 @@ def deliver_emails(once=False, send_email=True, thread=0, bulk=1000, delay=60, s
     sanity_check(executable=executable, hostname=hostname)
 
     # Make an initial heartbeat so that all daemons have the correct worker number on the next try
-    live(executable=executable, hostname=hostname, pid=pid, thread=heartbeat_thread)
+    heartbeat = live(executable=executable, hostname=hostname, pid=pid, thread=heartbeat_thread)
+    prepend_str = 'hermes-email [%i/%i] : ' % (heartbeat['assign_thread'], heartbeat['nr_threads'])
+    logger = formatted_logger(logging.log, prepend_str + '%s')
+
     GRACEFUL_STOP.wait(1)
 
     email_from = config_get('messaging-hermes', 'email_from')
 
     while not GRACEFUL_STOP.is_set():
-
         heartbeat = live(executable, hostname, pid, heartbeat_thread)
-        logging.debug('[email] %i:%i - bulk %i', heartbeat['assign_thread'],
-                      heartbeat['nr_threads'], bulk)
+        prepend_str = 'hermes-email [%i/%i] : ' % (heartbeat['assign_thread'], heartbeat['nr_threads'])
+        logger = formatted_logger(logging.log, prepend_str + '%s')
+        logger(logging.DEBUG, 'bulk %i', bulk)
 
         t_start = time.time()
 
@@ -101,8 +105,7 @@ def deliver_emails(once=False, send_email=True, thread=0, bulk=1000, delay=60, s
         if messages != []:
             to_delete = []
             for message in messages:
-                logging.debug('[email] %i:%i - submitting: %s', heartbeat['assign_thread'],
-                              heartbeat['nr_threads'], str(message))
+                logger(logging.DEBUG, 'submitting: %s', str(message))
 
                 if PY2:
                     msg = MIMEText(message['payload']['body'].encode('utf-8'))
@@ -128,30 +131,25 @@ def deliver_emails(once=False, send_email=True, thread=0, bulk=1000, delay=60, s
                                   'payload': str(message['payload']),
                                   'event_type': 'email'})
 
-                logging.debug('[email] %i:%i - submitting done: %s',
-                              heartbeat['assign_thread'], heartbeat['nr_threads'],
-                              str(message['id']))
+                logger(logging.DEBUG, 'submitting done: %s',
+                       str(message['id']))
 
             delete_messages(to_delete)
-            logging.info('[email] %i:%i - submitted %i messages',
-                         heartbeat['assign_thread'],
-                         heartbeat['nr_threads'], len(to_delete))
+            logger(logging.INFO, 'submitted %i messages', len(to_delete))
 
         if once:
             break
 
         if len(messages) < bulk:
-            logging.info("Only %d messages, which is less than the bulk %d, will sleep"
-                         % (len(messages), bulk))
+            logger(logging.INFO, "Only %d messages, which is less than the bulk %d, will sleep"
+                   % (len(messages), bulk))
             daemon_sleep(start_time=t_start, sleep_time=sleep_time, graceful_stop=GRACEFUL_STOP)
 
-    logging.debug('[email] %i:%i - graceful stop requested', heartbeat['assign_thread'],
-                  heartbeat['nr_threads'])
+    logger(logging.DEBUG, 'graceful stop requested')
 
     die(executable, hostname, pid, heartbeat_thread)
 
-    logging.debug('[email] %i:%i - graceful stop done', heartbeat['assign_thread'],
-                  heartbeat['nr_threads'])
+    logger(logging.DEBUG, 'graceful stop done')
 
 
 class HermesListener(stomp.ConnectionListener):
@@ -241,20 +239,18 @@ def deliver_messages(once=False, brokers_resolved=None, thread=0, bulk=1000, del
 
             heartbeat = live(executable=executable, hostname=hostname, pid=pid,
                              thread=heartbeat_thread)
+            prepend_str = 'broker [%i/%i] : ' % (heartbeat['assign_thread'], heartbeat['nr_threads'])
+            logger = formatted_logger(logging.log, prepend_str + '%s')
 
-            logging.debug('[broker] %i:%i - using: %s', heartbeat['assign_thread'],
-                          heartbeat['nr_threads'],
-                          [conn.transport._Transport__host_and_ports[0][0] for conn in conns])
+            logger(logging.DEBUG, 'using: %s', [conn.transport._Transport__host_and_ports[0][0] for conn in conns])
 
             messages = retrieve_messages(bulk=bulk,
                                          thread=heartbeat['assign_thread'],
                                          total_threads=heartbeat['nr_threads'])
 
             if messages:
-
-                logging.debug('[broker] %i:%i - retrieved %i messages',
-                              heartbeat['assign_thread'], heartbeat['nr_threads'],
-                              len(messages))
+                logger(logging.DEBUG, 'retrieved %i messages',
+                       len(messages))
                 to_delete = []
                 for message in messages:
                     try:
@@ -263,16 +259,12 @@ def deliver_messages(once=False, brokers_resolved=None, thread=0, bulk=1000, del
                             host_and_ports = conn.transport._Transport__host_and_ports[0][0]
                             RECONNECT_COUNTER.labels(host=host_and_ports.split('.')[0]).inc()
                             if not use_ssl:
-                                logging.info('[broker] %i:%i - connecting with USERPASS to %s',
-                                             heartbeat['assign_thread'],
-                                             heartbeat['nr_threads'],
-                                             host_and_ports)
+                                logger(logging.INFO, 'connecting with USERPASS to %s',
+                                       host_and_ports)
                                 conn.connect(username, password, wait=True)
                             else:
-                                logging.info('[broker] %i:%i - connecting with SSL to %s',
-                                             heartbeat['assign_thread'],
-                                             heartbeat['nr_threads'],
-                                             host_and_ports)
+                                logger(logging.INFO, 'connecting with SSL to %s',
+                                       host_and_ports)
                                 conn.connect(wait=True)
 
                         conn.send(body=json.dumps({'event_type': str(message['event_type']).lower(),
@@ -288,8 +280,8 @@ def deliver_messages(once=False, brokers_resolved=None, thread=0, bulk=1000, del
                                           'payload': json.dumps(message['payload']),
                                           'event_type': message['event_type']})
                     except ValueError:
-                        logging.warning('Cannot serialize payload to JSON: %s',
-                                        str(message['payload']))
+                        logger(logging.WARNING, 'Cannot serialize payload to JSON: %s',
+                               str(message['payload']))
                         to_delete.append({'id': message['id'],
                                           'created_at': message['created_at'],
                                           'updated_at': message['created_at'],
@@ -297,69 +289,61 @@ def deliver_messages(once=False, brokers_resolved=None, thread=0, bulk=1000, del
                                           'event_type': message['event_type']})
                         continue
                     except stomp.exception.NotConnectedException as error:
-                        logging.warning('Could not deliver message due to NotConnectedException: %s',
-                                        str(error))
+                        logger(logging.WARNING, 'Could not deliver message due to NotConnectedException: %s',
+                               str(error))
                         continue
                     except stomp.exception.ConnectFailedException as error:
-                        logging.warning('Could not deliver message due to ConnectFailedException: %s',
-                                        str(error))
+                        logger(logging.WARNING, 'Could not deliver message due to ConnectFailedException: %s',
+                               str(error))
                         continue
                     except Exception as error:
-                        logging.warning('Could not deliver message: %s', str(error))
-                        logging.critical(traceback.format_exc())
+                        logger(logging.WARNING, 'Could not deliver message: %s', str(error))
+                        logger(logging.CRITICAL, traceback.format_exc())
                         continue
 
                     if str(message['event_type']).lower().startswith('transfer') or str(message['event_type']).lower().startswith('stagein'):
-                        logging.debug('[broker] %i:%i - event_type: %s, scope: %s, name: %s, rse: %s, request-id: %s, transfer-id: %s, created_at: %s',
-                                      heartbeat['assign_thread'], heartbeat['nr_threads'],
-                                      str(message['event_type']).lower(),
-                                      message['payload'].get('scope', None),
-                                      message['payload'].get('name', None),
-                                      message['payload'].get('dst-rse', None),
-                                      message['payload'].get('request-id', None),
-                                      message['payload'].get('transfer-id', None),
-                                      str(message['created_at']))
+                        logger(logging.DEBUG, 'event_type: %s, scope: %s, name: %s, rse: %s, request-id: %s, transfer-id: %s, created_at: %s',
+                               str(message['event_type']).lower(),
+                               message['payload'].get('scope', None),
+                               message['payload'].get('name', None),
+                               message['payload'].get('dst-rse', None),
+                               message['payload'].get('request-id', None),
+                               message['payload'].get('transfer-id', None),
+                               str(message['created_at']))
 
                     elif str(message['event_type']).lower().startswith('dataset'):
-                        logging.debug('[broker] %i:%i - event_type: %s, scope: %s, name: %s, rse: %s, rule-id: %s, created_at: %s)',
-                                      heartbeat['assign_thread'],
-                                      heartbeat['nr_threads'],
-                                      str(message['event_type']).lower(),
-                                      message['payload']['scope'],
-                                      message['payload']['name'],
-                                      message['payload']['rse'],
-                                      message['payload']['rule_id'],
-                                      str(message['created_at']))
+                        logger(logging.DEBUG, 'event_type: %s, scope: %s, name: %s, rse: %s, rule-id: %s, created_at: %s)',
+                               str(message['event_type']).lower(),
+                               message['payload']['scope'],
+                               message['payload']['name'],
+                               message['payload']['rse'],
+                               message['payload']['rule_id'],
+                               str(message['created_at']))
 
                     elif str(message['event_type']).lower().startswith('deletion'):
                         if 'url' not in message['payload']:
                             message['payload']['url'] = 'unknown'
-                        logging.debug('[broker] %i:%i - event_type: %s, scope: %s, name: %s, rse: %s, url: %s, created_at: %s)',
-                                      heartbeat['assign_thread'],
-                                      heartbeat['nr_threads'],
-                                      str(message['event_type']).lower(),
-                                      message['payload']['scope'],
-                                      message['payload']['name'],
-                                      message['payload']['rse'],
-                                      message['payload']['url'],
-                                      str(message['created_at']))
+                        logger(logging.DEBUG, 'event_type: %s, scope: %s, name: %s, rse: %s, url: %s, created_at: %s)',
+                               str(message['event_type']).lower(),
+                               message['payload']['scope'],
+                               message['payload']['name'],
+                               message['payload']['rse'],
+                               message['payload']['url'],
+                               str(message['created_at']))
                     else:
-                        logging.debug('[broker] %i:%i - other message: %s',
-                                      heartbeat['assign_thread'], heartbeat['nr_threads'],
-                                      message)
+                        logger(logging.DEBUG, 'other message: %s',
+                               message)
 
                 delete_messages(to_delete)
-                logging.info('[broker] %i:%i - submitted %i messages',
-                             heartbeat['assign_thread'],
-                             heartbeat['nr_threads'],
-                             len(to_delete))
+                logger(logging.INFO, 'submitted %i messages',
+                       len(to_delete))
 
                 if once:
                     break
 
             if len(messages) < bulk:
-                logging.info("Only %d messages, which is less than the bulk %d, will sleep"
-                             % (len(messages), bulk))
+                logger(logging.INFO, "Only %d messages, which is less than the bulk %d, will sleep"
+                       % (len(messages), bulk))
                 daemon_sleep(start_time=t_start, sleep_time=sleep_time, graceful_stop=GRACEFUL_STOP)
 
         except NoResultFound:
