@@ -47,6 +47,8 @@ skip_rse_tests_with_accounts = pytest.mark.skipif(not any(os.path.exists(os.path
                                                   reason='fails if no rse-accounts.cfg found')
 skiplimitedsql = pytest.mark.skipif('RDBMS' in os.environ and os.environ['RDBMS'] == 'sqlite',
                                     reason="does not work in SQLite because of missing features")
+skip_multivo = pytest.mark.skipif('SUITE' in os.environ and os.environ['SUITE'] == 'multi_vo',
+                                  reason="does not work for multiVO")
 
 
 def get_long_vo():
@@ -94,13 +96,13 @@ def file_generator(size=2, namelen=10):
     return fn
 
 
-def make_temp_file(dir, data):
+def make_temp_file(dir_, data):
     """
     Creates a temporal file and write `data` on it.
     :param data: String to be writen on the created file.
     :returns: Name of the temporal file.
     """
-    fd, path = tempfile.mkstemp(dir=dir)
+    fd, path = tempfile.mkstemp(dir=dir_)
     if PY3:
         with os.fdopen(fd, 'w', encoding='utf-8') as f:
             f.write(data)
