@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2020-2021 CERN
+# Copyright 2020-2022 CERN
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -17,7 +17,9 @@
 # - Aristeidis Fkiaras <aristeidis.fkiaras@cern.ch>, 2020
 # - Benedikt Ziemons <benedikt.ziemons@cern.ch>, 2020-2021
 # - Simon Fayer <simon.fayer05@imperial.ac.uk>, 2021
+# - David Población Criado <david.poblacion.criado@cern.ch>, 2021
 # - Cedric Serfon <cedric.serfon@cern.ch>, 2021
+# - Rob Barnsley <rob.barnsley@skao.int>, 2022
 
 from copy import deepcopy
 
@@ -32,8 +34,10 @@ from rucio.common.types import InternalAccount, InternalScope
 from rucio.common.utils import generate_uuid
 from rucio.core.did import add_did, delete_dids, set_metadata_bulk, set_dids_metadata_bulk
 from rucio.core.did_meta_plugins import list_dids, get_metadata, set_metadata
+from rucio.core.did_meta_plugins.mongo_meta import MongoDidMeta
 from rucio.db.sqla.session import get_session
 from rucio.db.sqla.util import json_implemented
+from rucio.tests.common import skip_rse_tests_with_accounts
 from rucio.tests.common_server import get_vo
 
 
@@ -222,6 +226,98 @@ class TestDidMetaJSON(unittest.TestCase):
         assert [tmp_dsn4] == results
 
 
+@skip_rse_tests_with_accounts
+class TestDidMetaMongo(unittest.TestCase):
+
+    def setUp(self):
+        self.session = get_session()
+        if config_get_bool('common', 'multi_vo', raise_exception=False, default=False):
+            self.vo = {'vo': get_vo()}
+        else:
+            self.vo = {}
+        self.tmp_scope = InternalScope('mock', **self.vo)
+        self.root = InternalAccount('root', **self.vo)
+
+        self.mongo_meta = MongoDidMeta(
+            host='mongo',
+            port=27017,
+            db='test_db',
+            collection='test_collection'
+        )
+
+    def tearDown(self):
+        self.mongo_meta.drop_database()
+
+    @pytest.mark.dirty
+    def test_set_get_metadata(self):
+        """ DID Meta (MONGO): Get/set did meta """
+
+        did_name = 'mock_did_%s' % generate_uuid()
+        meta_key = 'my_key_%s' % generate_uuid()
+        meta_value = 'my_value_%s' % generate_uuid()
+        add_did(scope=self.tmp_scope, name=did_name, did_type='DATASET', account=self.root)
+        self.mongo_meta.set_metadata(scope=self.tmp_scope, name=did_name, key=meta_key, value=meta_value)
+        assert self.mongo_meta.get_metadata(scope=self.tmp_scope, name=did_name)[meta_key] == meta_value
+
+    @pytest.mark.dirty
+    def test_list_did_meta(self):
+        """ DID Meta (MONGO): List did meta """
+
+        meta_key1 = 'my_key_%s' % generate_uuid()
+        meta_key2 = 'my_key_%s' % generate_uuid()
+        meta_value1 = 'my_value_%s' % generate_uuid()
+        meta_value2 = 'my_value_%s' % generate_uuid()
+
+        tmp_dsn1 = 'dsn_%s' % generate_uuid()
+        add_did(scope=self.tmp_scope, name=tmp_dsn1, did_type="DATASET", account=self.root)
+        self.mongo_meta.set_metadata(scope=self.tmp_scope, name=tmp_dsn1, key=meta_key1, value=meta_value1)
+
+        tmp_dsn2 = 'dsn_%s' % generate_uuid()
+        add_did(scope=self.tmp_scope, name=tmp_dsn2, did_type="DATASET", account=self.root)
+        self.mongo_meta.set_metadata(scope=self.tmp_scope, name=tmp_dsn2, key=meta_key1, value=meta_value2)
+
+        tmp_dsn3 = 'dsn_%s' % generate_uuid()
+        add_did(scope=self.tmp_scope, name=tmp_dsn3, did_type="DATASET", account=self.root)
+        self.mongo_meta.set_metadata(scope=self.tmp_scope, name=tmp_dsn3, key=meta_key2, value=meta_value1)
+
+        tmp_dsn4 = 'dsn_%s' % generate_uuid()
+        add_did(scope=self.tmp_scope, name=tmp_dsn4, did_type="DATASET", account=self.root)
+        self.mongo_meta.set_metadata(scope=self.tmp_scope, name=tmp_dsn4, key=meta_key1, value=meta_value1)
+        self.mongo_meta.set_metadata(scope=self.tmp_scope, name=tmp_dsn4, key=meta_key2, value=meta_value2)
+
+        dids = self.mongo_meta.list_dids(self.tmp_scope, {meta_key1: meta_value1})
+        results = sorted(list(dids))
+
+        assert len(results) == 2
+        # assert sorted([{'scope': tmp_scope, 'name': tmp_dsn1}, {'scope': tmp_scope, 'name': tmp_dsn4}]) == sorted(results)
+        expected = sorted([tmp_dsn1, tmp_dsn4])
+        assert expected == results
+
+        dids = self.mongo_meta.list_dids(self.tmp_scope, {meta_key1: meta_value2})
+        results = []
+        for d in dids:
+            results.append(d)
+        assert len(results) == 1
+        # assert [{'scope': (tmp_scope), 'name': str(tmp_dsn2)}] == results
+        assert [tmp_dsn2] == results
+
+        dids = self.mongo_meta.list_dids(self.tmp_scope, {meta_key2: meta_value1})
+        results = []
+        for d in dids:
+            results.append(d)
+        assert len(results) == 1
+        # assert [{'scope': (tmp_scope), 'name': tmp_dsn3}] == results
+        assert [tmp_dsn3] == results
+
+        dids = self.mongo_meta.list_dids(self.tmp_scope, {meta_key1: meta_value1, meta_key2: meta_value2})
+        results = []
+        for d in dids:
+            results.append(d)
+        assert len(results) == 1
+        # assert [{'scope': (tmp_scope), 'name': tmp_dsn4}] == results
+        assert [tmp_dsn4] == results
+
+
 class TestDidMetaClient(unittest.TestCase):
 
     def setUp(self):
